searchNodes=[{"ref":"Mutex.html","title":"Mutex","type":"module","doc":"This is the main module in this application, it implements a mutex as a GenServer with a notification system to be able to await lock releases. See README.md for how to use."},{"ref":"Mutex.html#await/3","title":"Mutex.await/3","type":"function","doc":"Locks a key if it is available, or waits for the key to be freed before attempting again to lock it. Returns the lock or fails with a timeout. Due to the notification system, multiple attempts can be made to lock if multiple processes are competing for the key. So timeout will be at least for the passed amount of milliseconds, but may be slightly longer. Default timeout is 5000 milliseconds. If the timeout is reached, the caller process exists as in GenServer.call/3. More information in the timeouts section."},{"ref":"Mutex.html#await_all/2","title":"Mutex.await_all/2","type":"function","doc":"Awaits multiple keys at once. Returns once all the keys have been locked, timeout is :infinity. If two processes are trying to lock [:user_1, :user_2] and [:user_2, :user_3] at the same time, this function ensures that no deadlock can happen and that one process will eventually lock all the keys. More information at the end of the deadlocks section."},{"ref":"Mutex.html#child_spec/1","title":"Mutex.child_spec/1","type":"function","doc":"Returns a specification to start this module under a supervisor. See Supervisor."},{"ref":"Mutex.html#goodbye/1","title":"Mutex.goodbye/1","type":"function","doc":"Tells the mutex to release all the keys owned by the calling process and returns immediately with :ok."},{"ref":"Mutex.html#init/1","title":"Mutex.init/1","type":"function","doc":"Invoked when the server is started. start_link/3 or start/3 will block until it returns. init_arg is the argument term (second argument) passed to start_link/3. Returning {:ok, state} will cause start_link/3 to return {:ok, pid} and the process to enter its loop. Returning {:ok, state, timeout} is similar to {:ok, state}, except that it also sets a timeout. See the &quot;Timeouts&quot; section in the module documentation for more information. Returning {:ok, state, :hibernate} is similar to {:ok, state} except the process is hibernated before entering the loop. See c:handle_call/3 for more information on hibernation. Returning {:ok, state, {:continue, continue}} is similar to {:ok, state} except that immediately after entering the loop the c:handle_continue/2 callback will be invoked with the value continue as first argument. Returning :ignore will cause start_link/3 to return :ignore and the process will exit normally without entering the loop or calling c:terminate/2. If used when part of a supervision tree the parent supervisor will not fail to start nor immediately try to restart the GenServer. The remainder of the supervision tree will be started and so the GenServer should not be required by other processes. It can be started later with Supervisor.restart_child/2 as the child specification is saved in the parent supervisor. The main use cases for this are: The GenServer is disabled by configuration but might be enabled later. An error occurred and it will be handled by a different mechanism than the Supervisor. Likely this approach involves calling Supervisor.restart_child/2 after a delay to attempt a restart. Returning {:stop, reason} will cause start_link/3 to return {:error, reason} and the process to exit with reason reason without entering the loop or calling c:terminate/2. Callback implementation for GenServer.init/1."},{"ref":"Mutex.html#lock/2","title":"Mutex.lock/2","type":"function","doc":"Attemps to lock a resource on the mutex and returns immediately with the result, which is either a Mutex.Lock structure or {:error, :busy}."},{"ref":"Mutex.html#lock!/2","title":"Mutex.lock!/2","type":"function","doc":"Attemps to lock a resource on the mutex and returns immediately with the lock or raises an exception if the key is already locked."},{"ref":"Mutex.html#release/2","title":"Mutex.release/2","type":"function","doc":"Tells the mutex to free the given lock and immediately returns :ok without waiting for the actual release. If the calling process is not the owner of the key(s), the key(s) is/are not released and an error is logged."},{"ref":"Mutex.html#start/1","title":"Mutex.start/1","type":"function","doc":"Starts a mutex with no process linking. Given options are passed as options for a GenServer, it&#39;s a good place to set the name for registering the process. See GenServer options. A :meta key can also be given in options to set the mutex metadata."},{"ref":"Mutex.html#start_link/1","title":"Mutex.start_link/1","type":"function","doc":"Starts a mutex with linking under a supervision tree. Given options are passed as options for a GenServer, it&#39;s a good place to set the name for registering the process. See GenServer options. A :meta key can also be given in options to set the mutex metadata."},{"ref":"Mutex.html#under/4","title":"Mutex.under/4","type":"function","doc":"Awaits a lock for the given key, executes the given fun and releases the lock immediately. If an exeption is raised or thrown in the fun, the lock is automatically released. If a function of arity 1 is given, it will be passed the lock. Otherwise the arity must be 0. You should not manually release the lock within the function."},{"ref":"Mutex.html#under_all/3","title":"Mutex.under_all/3","type":"function","doc":"Awaits a lock for the given keys, executes the given fun and releases the lock immediately. If an exeption is raised or thrown in the fun, the lock is automatically released. If a function of arity 1 is given, it will be passed the lock. Otherwise the arity must be 0. You should not manually release the lock within the function."},{"ref":"Mutex.html#t:key/0","title":"Mutex.key/0","type":"type","doc":"A key can be any term."},{"ref":"Mutex.html#t:name/0","title":"Mutex.name/0","type":"type","doc":"The name of a mutex is an atom, registered with Process.register/2"},{"ref":"Mutex.Lock.html","title":"Mutex.Lock","type":"module","doc":"This module defines a struct containing the key(s) locked with all the locking functions in Mutex."},{"ref":"Mutex.Lock.html#get_meta/1","title":"Mutex.Lock.get_meta/1","type":"function","doc":"Returns the metadata associated with the lock. The metadata is given to the mutex on initialization: Mutex.start_link(name: MyMutex, meta: metadata)"},{"ref":"Mutex.Lock.html#t:t/0","title":"Mutex.Lock.t/0","type":"type","doc":"The struct containing the key(s) locked during a lock operation. :type specifies wether there is one or more keys."},{"ref":"readme.html","title":"A simple mutex for Elixir.","type":"extras","doc":"A simple mutex for Elixir. Mutex is a simple mutex module that fits under your supervision tree and allows processes to work on shared ressources one by one. This can be a simple alternative to database transactions. Also, Mutex supports multiple keys locking without deadlocks."},{"ref":"readme.html#installation","title":"A simple mutex for Elixir. - Installation","type":"extras","doc":"This package can be installed by adding mutex to your list of dependencies in mix.exs: def deps do [{:mutex, &quot;~&gt; 1.1.0&quot;}] end"},{"ref":"readme.html#using-mutex","title":"A simple mutex for Elixir. - Using Mutex","type":"extras","doc":"A mutex is handled by a process that you start in your supervision tree with child_spec(name). Options can be an atom (used as the GenServer name), or a Keyword with GenServer options and a :meta option to set the metadata. children = [ {Mutex, name: MyMutex, meta: some_data} ] {:ok, _pid} = Supervisor.start_link(children, strategy: :one_for_one) Let&#39;s see a bad example of concurrent code without database transactions. resource_id = :some_user_id # unused in example update_user = fn(worker) -&gt; IO.puts &quot;[\#{worker}] Reading user from database.&quot; Process.sleep(250) IO.puts &quot;[\#{worker}] Working with user.&quot; Process.sleep(250) IO.puts &quot;[\#{worker}] Saving user in database.&quot; Process.sleep(500) end spawn(fn -&gt; update_user.(&quot;worker 1&quot;) end) spawn(fn -&gt; update_user.(&quot;worker 2&quot;) end) spawn(fn -&gt; update_user.(&quot;worker 3&quot;) end) # Results : # [worker 1] Reading user from database. # [worker 2] Reading user from database. # [worker 3] Reading user from database. # [worker 1] Working with user. # [worker 2] Working with user. # [worker 3] Working with user. # [worker 2] Saving user in database. # 2 before 1 ! # [worker 1] Saving user in database. # [worker 3] Saving user in database. Serialized transactions could be a good fit for this specific case, but let&#39;s see another simple solution : With a simple mutex mechanism, workers will be able to wait until the resource is saved in database before loading it, with the guarantee that any other worker will not be able to touch the resource. Each worker will wait for the key identifying the resource to be available on the mutex and attempt to lock it. A key can be anything : :my_resource, {User, user_id}, &quot;http://example.com&quot; or more complex structures. When the lock is eventually acquired, the worker can work with the resource. Of course, this works only if all your workers use the mutex, with the same key. resource_id = {User, {:id, 1}} update_user = fn(worker) -&gt; lock = Mutex.await(MyMutex, resource_id) IO.puts &quot;[\#{worker}] Reading user from database.&quot; Process.sleep(250) IO.puts &quot;[\#{worker}] Working with user.&quot; Process.sleep(250) IO.puts &quot;[\#{worker}] Saving user in database.&quot; Mutex.release(MyMutex, lock) end spawn(fn -&gt; update_user.(&quot;worker 4&quot;) end) spawn(fn -&gt; update_user.(&quot;worker 5&quot;) end) spawn(fn -&gt; update_user.(&quot;worker 6&quot;) end) # [worker 4] Reading user from database. # [worker 4] Working with user. # [worker 4] Saving user in database. # [worker 5] Reading user from database. # [worker 5] Working with user. # [worker 5] Saving user in database. # [worker 6] Reading user from database. # [worker 6] Working with user. # [worker 6] Saving user in database."},{"ref":"readme.html#error-handling","title":"A simple mutex for Elixir. - Error Handling","type":"extras","doc":"Whenever a process that locked a key on the mutex crashes, the mutex automatically unlocks the key so any other process can lock it in its turn. But if you catch exceptions you may forget to release the keys and keep unnecessary keys locked for a while : # Do not do this try do lock = Mutex.await(MyMutex, :some_key) throw(:fail) # This will never happen: Mutex.release(MyMutex, lock) catch :throw, :fail -&gt; :ok end Whenever possible, avoid to lock keys in try, if, for, ... blocks. The mutex provides a mechanism to automatically handle this situation. Using Mutex.under/3, the calling process will wait for the key to be available and lock it. Then the passed fn will be executed and if it raises or throws, the lock will automatically be removed. Exceptions and thrown values are reraised and rethrown so you still have to handle them. # Do this instead try do Mutex.under(MyMutex, :some_key, fn -&gt; throw(:fail) end) catch :throw, :fail -&gt; :ok end A multilock version is also available with Mutex.under_all/3."},{"ref":"readme.html#avoiding-deadlocks","title":"A simple mutex for Elixir. - Avoiding Deadlocks","type":"extras","doc":"A deadlock would occur if the keys were locked one by one with a race condition : # Do not do this def handle_order(buyer, seller) do lock1 = Mutex.await(MyMutex, buyer) lock2 = Mutex.await(MyMutex, seller) do_some_work_with_users(buyer, seller) Mutex.release(MyMutex, lock1) Mutex.release(MyMutex, lock2) end spawn(fn -&gt; handler_order(:user_1, :user_2) end) # Process 1 spawn(fn -&gt; handler_order(:user_2, :user_1) end) # Process 2 Process 1 will first lock :user_1 and process 2 will lock :user_2, and then each process is waiting for the key that is already locked by the other one. If any process should have, at any given time, several keys locked, those keys shall have been locked all at once. This simple rule is mandatory and sufficient to be free from deadlocks, and Mutex.await_all/2 is the simplest way to respect that rule. # Do this instead def handle_order(buyer, seller) do lock = Mutex.await_all(MyMutex, [buyer, seller]) do_some_work_with_users(buyer, seller) Mutex.release(MyMutex, lock) end If you really have to lock keys in a loop, or in mutiple moments, the Mutex.goodbye/1 function allows to simply release all the keys locked by the calling process in one call."},{"ref":"readme.html#metadata","title":"A simple mutex for Elixir. - Metadata","type":"extras","doc":"A mutex can hold metadata that will be assigned to each lock. The metadata is set upon initialization (given as :meta in the child spec in your supervisor, or in the options for Mutex.start or Mutex.start_link). The metadata is sent to any client that locks a key or a group of keys: {:ok, pid} = Mutex.start_link(meta: :some_data) {:ok, lock} = Mutex.lock(pid, :some_key) lock.meta === :some_data Mutex.Lock.get_meta(lock) === :some_data The lock will also be passed to a fun if its arity is 1 when using Mutex.under/4 and Mutex.under_all/3. The arity of the fun can also be 0. Releasing the lock within the fun is still useless as it will be automatically release as for 0-arity funs, and could give other processes the ability to lock the keys before the fun execution is complete."}]